
class VarSiblings:
    def __init__(self):
        pass

    def value(self, filename: str):
        # @todo #141:30min: Implement var_siblings pattern.
        #  This pattern must return a tuple containing the line numbers of the two siblings. After implementing that
        #  activate the tests in test_car_sibling.py
        return []
